/* PSG2YM.C by Raul Gomez Sanchez.
 *
 *   This program converts from PSG sound files saved by R80 Spectrum Emulator
 *   to YM3 files. How to use:
 *
 *                psg2ym <source.PSG> <destination.YM>
 *
 *  Do what you like with this code. I'm not responsible of anything
 *  this program does, use at your own risk :).
 */

#include <stdio.h>

#define MAX_SIZE 100000 // it has room for about 33 minutes. If you need
                        // more, change and compile yourself.

unsigned char r0[MAX_SIZE];
unsigned char r1[MAX_SIZE];
unsigned char r2[MAX_SIZE];
unsigned char r3[MAX_SIZE];
unsigned char r4[MAX_SIZE];
unsigned char r5[MAX_SIZE];
unsigned char r6[MAX_SIZE];
unsigned char r7[MAX_SIZE];
unsigned char r8[MAX_SIZE];
unsigned char r9[MAX_SIZE];
unsigned char r10[MAX_SIZE];
unsigned char r11[MAX_SIZE];
unsigned char r12[MAX_SIZE];
unsigned char r13[MAX_SIZE];
int puntero=0,punterof=1,punteroini=0;

FILE *source;
FILE *destination;

int main(int argc, char *argv[]) {
	unsigned char dato,valor;
  unsigned char ym3header[4]="YM3!";

  printf("PSG2YM v1.0 <> by Raul Gomez Sanchez (raulgomez@mx3.redestb.es)\n");
	printf("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<\n");

  // check parameters
 	if (argc != 3) {
   	printf("\nUse: psg2ym <source.psg> <destination.ym>\n");
    return(1);
  }

  // open source file
  if ((source=fopen(argv[1],"rb")) == NULL) {
   	printf("\nSource file %s does not exists\n",argv[1]);
    return(1);
  }

  // open destination file
  destination=fopen(argv[2],"wb");

  // initialize registers
	r0[0]=0x00;	r1[0]=0x00;	r2[0]=0x00;	r3[0]=0x00;	r4[0]=0x00;	r5[0]=0x00;
	r6[0]=0x00;	r7[0]=0xff;	r8[0]=0x00;	r9[0]=0x00;	r10[0]=0x00; r11[0]=0x00;
	r12[0]=0x00; r13[0]=0xff;

  // read file
  printf("Reading %s\n",argv[1]);
  while (fread(&dato,1,1,source) == 1) {
  	if (dato != 0xff) {
			fread(&valor,1,1,source);
      fseek(source,4,SEEK_CUR);
    }
   	switch (dato) {
     	case 0: r0[punterof]=valor; break;
     	case 1: r1[punterof]=valor; break;
     	case 2: r2[punterof]=valor; break;
      case 3: r3[punterof]=valor; break;
      case 4: r4[punterof]=valor; break;
      case 5: r5[punterof]=valor; break;
      case 6: r6[punterof]=valor; break;
      case 7: r7[punterof]=valor; break;
      case 8: r8[punterof]=valor; break;
      case 9: r9[punterof]=valor; break;
      case 10: r10[punterof]=valor; break;
      case 11: r11[punterof]=valor; break;
      case 12: r12[punterof]=valor; break;
      case 13: r13[punterof]=valor; break;
      case 0xff:
      	punterof++;
        r0[punterof]=r0[punterof-1];
        r1[punterof]=r1[punterof-1];
        r2[punterof]=r2[punterof-1];
        r3[punterof]=r3[punterof-1];
        r4[punterof]=r4[punterof-1];
        r5[punterof]=r5[punterof-1];
        r6[punterof]=r6[punterof-1];
        r7[punterof]=r7[punterof-1];
        r8[punterof]=r8[punterof-1];
        r9[punterof]=r9[punterof-1];
        r10[punterof]=r10[punterof-1];
        r11[punterof]=r11[punterof-1];
        r12[punterof]=r12[punterof-1];
        r13[punterof]=0xff;
      	break;
    }
  }

  // remove silence from beginning of file
  while (((r7[punteroini] & 0x3f) == 0x3f) ||
  	(((r8[punteroini] & 0x0f)) == 0x00)  &&
	  (((r9[punteroini] & 0x0f)) == 0x00)  &&
	  (((r10[punteroini] & 0x0f)) == 0x00)) {
   	punteroini++;
  }

  // remove silence from end of file
  while (((r7[punterof] & 0x3f) == 0x3f) ||
  	(((r8[punterof] & 0x0f)) == 0x00)  &&
	  (((r9[punterof] & 0x0f)) == 0x00)  &&
	  (((r10[punterof] & 0x0f)) == 0x00)) {
   	punterof--;
  }

  printf("Writing %s\n",argv[2]);

  // write file
  fwrite(ym3header,1,4,destination);
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r0[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r1[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r2[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r3[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r4[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r5[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r6[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r7[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r8[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r9[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r10[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r11[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r12[puntero];
   	fwrite(&dato,1,1,destination);
  }
  for (puntero=punteroini; puntero < punterof; puntero++) {
  	dato=r13[puntero];
   	fwrite(&dato,1,1,destination);
  }

  return(0);
}
